package dev.pfaff.altargui.natives;

import dev.pfaff.dyl4j.BundledLibraryLinker;
import dev.pfaff.glass.Arch;
import dev.pfaff.glass.OS;
import org.jspecify.annotations.NonNull;

import java.io.IOException;
import java.io.InputStream;
import java.lang.foreign.Arena;
import java.lang.foreign.SymbolLookup;

public final class Natives {
	private static final String PREFIX = "/dev/pfaff/altargui/natives";

	public static String getLibImageResource(OS os, Arch arch) {
		return switch (os) {
			case OS.Linux _ when arch instanceof Arch.aarch64 -> PREFIX + "/libimage_c-linux-arm64.so";
			case OS.macOS _ when arch instanceof Arch.amd64 -> PREFIX + "/libimage_c-macos-amd64.dylib";
			case OS.Windows _ when arch instanceof Arch.amd64 -> PREFIX + "/image_c-windows-amd64.dll";
			default -> throw dev.pfaff.glass.Host.makeUnsupportedOSArchException();
		};
	}

	public static SymbolLookup loadLibImage() {
		InputStream inputStream = Natives.class.getResourceAsStream(getLibImageResource(OS.HOST, Arch.HOST));
		assert inputStream != null;
		try {
			return BundledLibraryLinker.createLookup(inputStream, Arena.ofAuto());
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
